﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization; //for NumberStyles format specifiers

using AIOUSBNet;  // namespace exposes AIOUSB Class interface 

// you must also add a reference to the AIOUSBNet.dll in the project
// The path should be C:\Windows\system32\AIOUSBNet.dll or the equivalent for your system


namespace Sample1
{
    public partial class FormSample1 : Form
    {
        // Array of controls for GUI updates:
        public CheckBox[] checkOutArray = new CheckBox[16];
        public CheckBox[] checkInArray = new CheckBox[16];
        public TextBox[] txtWriteArray = new TextBox[2];

        public UInt32 DeviceIndex;

        // Interval Timer for GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();

        public FormSample1()
        {
            InitializeComponent();
            
            // Additional Constructor Initializations here:

            txtWriteArray[0] = txtWriteByte0;
            txtWriteArray[1] = txtWriteByte1;

            // Array of the LED style checkboxes:
            checkOutArray[0] = checkOutA0;
            checkOutArray[1] = checkOutA1;
            checkOutArray[2] = checkOutA2;
            checkOutArray[3] = checkOutA3;
            checkOutArray[4] = checkOutA4;
            checkOutArray[5] = checkOutA5;
            checkOutArray[6] = checkOutA6;
            checkOutArray[7] = checkOutA7;
            checkOutArray[8] = checkOutB8;
            checkOutArray[9] = checkOutB9;
            checkOutArray[10] = checkOutB10;
            checkOutArray[11] = checkOutB11;
            checkOutArray[12] = checkOutB12;
            checkOutArray[13] = checkOutB13;
            checkOutArray[14] = checkOutB14;
            checkOutArray[15] = checkOutB15;
            
            checkInArray[0] = checkInA0;
            checkInArray[1] = checkInA1;
            checkInArray[2] = checkInA2;
            checkInArray[3] = checkInA3;
            checkInArray[4] = checkInA4;
            checkInArray[5] = checkInA5;
            checkInArray[6] = checkInA6;
            checkInArray[7] = checkInA7;
            checkInArray[8] = checkInB8;
            checkInArray[9] = checkInB9;
            checkInArray[10] = checkInB10;
            checkInArray[11] = checkInB11;
            checkInArray[12] = checkInB12;
            checkInArray[13] = checkInB13;
            checkInArray[14] = checkInB14;
            checkInArray[15] = checkInB15;
        }
        //**************************************************************************************************
        // A few helper functions

        private bool HandleTryParseResult(bool bResult)
        {
            if (!bResult)
            {
                txtStatus.Text = "Format: Enter a hex number 00-FF valid hex values 0-9, A-F and a-f, no 0x prefix!";
                return false;
            }
            else
                txtStatus.Text = "";

            return true;
        }

        private void WriteHexStringToDIOAll(string HexString)
        {
            UInt32 bits = 0; // note, this changes based on how many bytes of DIO the card supports
            if (HandleTryParseResult(UInt32.TryParse(HexString, NumberStyles.HexNumber, null as IFormatProvider, out bits)))
                AIOUSB.DIO_WriteAll(DeviceIndex, ref bits);
        }

        private void WriteHexStringToDIOByte(string HexString, byte ByteIndex)
        {
            Byte bits = 0;
            if (HandleTryParseResult(Byte.TryParse(HexString, NumberStyles.HexNumber, null as IFormatProvider, out bits)))
                AIOUSB.DIO_Write8(DeviceIndex, ByteIndex, bits);
        }
        // end of helper functions
        //**************************************************************************************************

        private void FormSample1_Load(object sender, EventArgs e)
        {
            const int ERROR_SUCCESS = 0;
            UInt32 Status;
            
            // Query Device Info:
            UInt32 PID = 0;
            UInt32 NameSize = 256; // must pass this in by reference it gets used as sizeof buffer in and then modified as sizeof buffer out
            string strName = "name";
            UInt32 DIOBytes = 0;
            UInt32 Counters = 0;

            bool deviceIndexValid = false;
            DeviceIndex = AIOUSB.diFirst;   // Default to first device found

            do
            {
                Status = AIOUSB.QueryDeviceInfo(DeviceIndex, out PID, ref NameSize, out strName, out DIOBytes, out Counters);

                // all USB-IIRO" and -IDIO" boards fall in this range
                if (Status == ERROR_SUCCESS && PID >= 0x8010 && PID <= 0x801F)
                    deviceIndexValid = true; // yes, this sample is for the detected device.
                else
                {
                    // If First device is not useful for this sample, Launch connect device dialog box:
                    FormDetect DetectSubForm = new FormDetect(this);
                    DetectSubForm.ShowDialog();
                }

            } while (deviceIndexValid == false);


            if (!deviceIndexValid)  // No valid device found, should exit:
                this.Close();


            if (Status == ERROR_SUCCESS)  // Display device Status:
            {
                string formatString = "Index: {0,8:X8}" + " PID: {1,8:X8}" + " Name: {2}";;
                txtStatus.Text = " Device Connected !  " + String.Format(formatString, DeviceIndex, PID, strName);
            }
            else
                txtStatus.Text = " Not Connected ! ";

            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            myTimer.Tick += new EventHandler(TimerEventProcessor);

            // Set the timer interval in miliseconds and start  55ms min resolution:
            myTimer.Interval = 100;
            myTimer.Start();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            // Right now this is only called on button click not window close ALtF4 etc:
            
            // Clean up and Close App:
            UInt32 Status;

            Status = AIOUSB.ClearDevices();

            // Kill the timer:
            myTimer.Stop();
            myTimer.Enabled = false;
            myTimer.Dispose();

            // Close the form and Application:
            this.Close();
        }
  
        private void btnDevice_Click(object sender, EventArgs e)
        {
            // Launch connect device dialog box
            // Switch between multiple devices or redetect

            FormDetect DetectSubForm = new FormDetect(this);
            DetectSubForm.ShowDialog();
        }

        private void btnWritePortAll_Click(object sender, EventArgs e)
        {
            WriteHexStringToDIOAll(txtWriteAll.Text);
        }

          private void btnOutRelay_Click(object sender, EventArgs e)
        {
            // Button Clicks to toggle the relay and hence display states:
            byte stateByte = 1;
            UInt32 bits = 0; // Note, this would change for device-firmwares with different numbers of DIO bytes

            // Get sender's Tag (bitNumber)
            byte bitNumber = Convert.ToByte((sender as Button).Tag);

            // Read All then test correct bit and set individual relay state:
            AIOUSB.DIO_ReadAll(DeviceIndex, out bits);

            if ((bits & (1 << bitNumber)) != 0)
                stateByte = 0;
            else
                stateByte = 1;

            AIOUSB.DIO_Write1(DeviceIndex, bitNumber, stateByte);
            
            // NOTE:
            // You can also use DIO_WriteAll(), which is much faster than even just two writes to DIO_Write1:
            //      result = AIOUSB.DIO_ReadAll(DeviceIndex, out bits);
            //      result = AIOUSB.DIO_WriteAll(DeviceIndex, bits ^ (1 << bitNumber)); // toggle using XOR
            // A single call to DIO_Write1() is almost exactly the same speed (actually a few cpu cycles SLOWER)
            // than a single call to DIO_WriteAll() -- but DIO_Write1 is often EASIER
        }

        private void btnWritePortByte_Click(object sender, EventArgs e)
        {
            Byte portIndex = Convert.ToByte( (sender as Button).Tag);
            WriteHexStringToDIOByte(txtWriteArray[portIndex].Text,  portIndex); 
        }

        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            // This is the method that runs when the timer event is raised:           
            UInt32 result;
            UInt32 bits = 0;
            string tempstring;

            myTimer.Stop();

            result = AIOUSB.DIO_ReadAll(DeviceIndex, out bits);

            // Display Hex format:
            tempstring = bits.ToString("X8");
            txtReadAllHex.Text = tempstring;
            txtReadByte3.Text = tempstring.Substring(0, 2);
            txtReadByte2.Text = tempstring.Substring(2, 2);

            // Display Binary format:
            txtReadAllBin.Text = Convert.ToString(bits, 2).PadLeft(32, '0');

            // Cycle through the A inputs (and outputs) and set the display to the right state:
            for (int i = 0; i < 16; i++)
            {
                if ((bits & (1 << (i))) != 0)
                    checkOutArray[i].Checked = true;
                else
                    checkOutArray[i].Checked = false;

                if ((bits & (1 << (i + 16))) != 0)
                    checkInArray[i].Checked = true;
                else
                    checkInArray[i].Checked = false;
            }

            myTimer.Enabled = true;
        }
    }
}
